@@ -20,7 +20,7 @@ build_time_vars = {'ABIFLAGS': '',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'i386-unknown-openbsd7.6',
+ 'BUILD_GNU_TYPE': 'i386-unknown-openbsd7.7',
  'BUILD_SCRIPTS_DIR': 'build/scripts-3.12',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
@@ -60,9 +60,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.12',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/pobj/Python-3.12.8/Python-3.12.8/coverage.info',
+ 'COVERAGE_INFO': '/pobj/Python-3.12.9/Python-3.12.9/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/pobj/Python-3.12.8/Python-3.12.8/lcov-report',
+ 'COVERAGE_REPORT': '/pobj/Python-3.12.9/Python-3.12.9/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.12 LCOV report [commit $(shell '
                             ')]"',
@@ -539,6 +539,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_SYS_MODEM_H': 0,
  'HAVE_SYS_NDIR_H': 0,
  'HAVE_SYS_PARAM_H': 1,
+ 'HAVE_SYS_PIDFD_H': 0,
  'HAVE_SYS_POLL_H': 1,
  'HAVE_SYS_RANDOM_H': 0,
  'HAVE_SYS_RESOURCE_H': 1,
@@ -572,7 +573,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_TMPNAM_R': 0,
  'HAVE_TM_ZONE': 1,
  'HAVE_TRUNCATE': 1,
- 'HAVE_TTYNAME': 1,
+ 'HAVE_TTYNAME_R': 1,
  'HAVE_TZNAME': 0,
  'HAVE_UMASK': 1,
  'HAVE_UNAME': 1,
@@ -607,17 +608,17 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'i386-unknown-openbsd7.6',
+ 'HOST_GNU_TYPE': 'i386-unknown-openbsd7.7',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.12 '
                    '/usr/local/include/python3.12',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.12',
- 'INSTALL': '/pobj/Python-3.12.8/bin/install -c',
- 'INSTALL_DATA': '/pobj/Python-3.12.8/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/pobj/Python-3.12.8/bin/install -c -s -m 755',
- 'INSTALL_SCRIPT': '/pobj/Python-3.12.8/bin/install -c -m 755',
- 'INSTALL_SHARED': '/pobj/Python-3.12.8/bin/install -c -m 755',
+ 'INSTALL': '/pobj/Python-3.12.9/bin/install -c',
+ 'INSTALL_DATA': '/pobj/Python-3.12.9/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/pobj/Python-3.12.9/bin/install -c -s -m 755',
+ 'INSTALL_SCRIPT': '/pobj/Python-3.12.9/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/pobj/Python-3.12.9/bin/install -c -m 755',
  'INSTSONAME': 'libpython3.12.so.0.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -1054,15 +1055,15 @@ build_time_vars = {'ABIFLAGS': '',
                    '-I/usr/local/include -I./Include/internal -I. -I./Include '
                    '-I/usr/local/include -fPIC -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L/usr/local/lib/ -flto=thin -g '
-                    '-L/pobj/Python-3.12.8/Python-3.12.8 -L/usr/local/lib/',
+                    '-L/pobj/Python-3.12.9/Python-3.12.9 -L/usr/local/lib/',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/local/include',
  'PY_ENABLE_SHARED': 1,
  'PY_HAVE_PERF_TRAMPOLINE': 0,
  'PY_LDFLAGS': '-L/usr/local/lib/',
- 'PY_LDFLAGS_NODIST': '-flto=thin -g -L/pobj/Python-3.12.8/Python-3.12.8 '
+ 'PY_LDFLAGS_NODIST': '-flto=thin -g -L/pobj/Python-3.12.9/Python-3.12.9 '
                       '-L/usr/local/lib/',
  'PY_LDFLAGS_NOLTO': '-L/usr/local/lib/ -flto=thin '
-                     '-L/pobj/Python-3.12.8/Python-3.12.8 -L/usr/local/lib/',
+                     '-L/pobj/Python-3.12.9/Python-3.12.9 -L/usr/local/lib/',
  'PY_SQLITE_ENABLE_LOAD_EXTENSION': 1,
  'PY_SQLITE_HAVE_SERIALIZE': 1,
  'PY_SSL_DEFAULT_CIPHERS': 1,
@@ -1087,7 +1088,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': '@READELF@',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/pobj/Python-3.12.8/Python-3.12.8',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/pobj/Python-3.12.9/Python-3.12.9',
  'SCRIPTDIR': '/usr/local/lib',
  'SCRIPT_2TO3': 'build/scripts-3.12/2to3-3.12',
  'SCRIPT_IDLE': 'build/scripts-3.12/idle3.12',
@@ -1192,9 +1193,9 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/pobj/Python-3.12.8/Python-3.12.8 ./python -E',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/pobj/Python-3.12.9/Python-3.12.9 ./python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/pobj/Python-3.12.8/Python-3.12.8 ./python -E '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/pobj/Python-3.12.9/Python-3.12.9 ./python -E '
                './Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'idlelib/idle_test \\',
  'TESTTIMEOUT': 1200,
@@ -1224,8 +1225,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 1,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/pobj/Python-3.12.8/Python-3.12.8',
- 'abs_srcdir': '/pobj/Python-3.12.8/Python-3.12.8',
+ 'abs_builddir': '/pobj/Python-3.12.9/Python-3.12.9',
+ 'abs_srcdir': '/pobj/Python-3.12.9/Python-3.12.9',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
